<?php

namespace Ecommerce\Util;

use Teknisa\Libs\Util\SMTP;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class EmailController
{
    protected $environment;
    protected $connection;
    protected $fileServerURL;
    protected $smtpUtil;
    protected $libUtilities;
    
    private $appId;
    private $unitId;
    private $useTemplate;
    private $isUsingCustomSmtp = false;
    private $customSmtpSettings = array();
    
    public function __construct($environment, $connection, $fileServerURL, $libUtilities) {
        $this->environment = $environment;
        $this->connection  = $connection;
        $this->fileServerURL = $fileServerURL;
        $this->libUtilities = $libUtilities;
        $this->directory = realpath(__DIR__ . '/Template');
        
        $this->appId = "PORTAL E-COMMERCE";
        $this->unitId = $this->environment->getCdFilial();
        $this->useTemplate = true;
	}
	
	public function send($subject, $content, $emails, $isUsingCustomSmtp = true)
    {
        try {
            if ($this->useTemplate) {
                $unitId = $this->unitId;

                $logo = $this->getLogo($this->appId);
                $lgpd = $this->getLgpd($unitId);

                $imgAttr = "width='200px'";
                $this->getLogoAttributes($logo, $imgAttr);
                
                $contentHeader = "<p class='bold' style='text-align: center; color: #ED2424'>" . $subject . "</p>"; 
                
                $file = array("name" => "email", "extension" => ".html");
                $replace = array(
                    "%ATTRIMG%" => $imgAttr, 
                    "%LOGOSRC%" => $logo, 
                    "%SUBJECT%" => $contentHeader, 
                    "%MESSAGE%" => $content, 
                    "%LGPD%" => $lgpd
                );
                $content = $this->getTemplate($file, $replace);
            }
            
            $this->isUsingCustomSmtp = $isUsingCustomSmtp;
            
            $this->useCustomSmtp(); 
            
            $this->smtpUtil->setSubject($subject);
            $this->smtpUtil->sendMail($content, $emails);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function useTemplate($bool)
    {
        $this->useTemplate = $bool;
    }
    
    public function setAppId($id)
    {
        $this->appId = $id;
    }
    
    public function setUnitId($id)
    {
        $this->unitId = $id;
    }
    
    public function getTemplate($file, $search_replace = array())
    {
        try {
            if ($file["name"]) {
                $path = $this->getFilePath($file);
                $content = $this->getFileContents($path);

                if (!empty($search_replace)) {
                    foreach ($search_replace as $index => $value) {
                        $search[] = $index;
                        $replace[] = $value;
                    }
                    
                    $content = $this->replaceTags($search, $replace, $content);
                }
                
                return $content;
            } else {
                throw new \Exception("Não foi enviado o nome do arquivo.", 400);
            }
        } catch (Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
    
    private function getFilePath($file)
    {
        $fileName = $file["name"];
        $fileExtension = isset($file["extension"]) ? $file["extension"] : ".html";
        $filePath = $this->directory . '/' . $fileName . $fileExtension;
        
        if (file_exists($filePath)) {
            return $filePath;
        } else {
            throw new \Exception("Caminho de arquivo não encontrado.", 400);
        }
    }
    
    private function getFileContents($path)
    {
        if (file_exists($path)) {
            $content = file_get_contents($path);
            return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
        } else {
            throw new \Exception("Caminho de arquivo não encontrado.", 400);
        }
    }
    
    private function replaceTags($search, $replace, $content)
    {
        $content = str_replace($search, $replace, $content, $count);

        if ($count > 0) {
            return $content;
        } else {
            throw new \Exception("Nenhuma alteração das tags foi feita. Verifique os arrays enviados.", 400);
        }
    }
    
    private function getLgpd($unitId)
    {
        $sql = "SELECT 
                    P.TXAVISOLEGALEMAIL AS LGPD
                FROM 
                    PARASUPRI P, 
                    FILIAL F 
                WHERE 
                    F.CDFILIAL = :P_CDFILIAL AND 
                    P.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
        
        $stmt = $this->connection->prepare($sql);
        
        $stmt->execute(
            array(
                "P_CDFILIAL" => $unitId
            )
        );
        
		$result = $stmt->fetchAll();
        
        return isset($result[0]) ? $result[0]['LGPD'] : "";
    }
    
    private function getLogo($appId) 
    {
        $sql = "SELECT 
                    URLLOGO AS LOGO
                FROM 
                    APPCONFIG 
                WHERE 
                    IDAPLICATIVO = :IDAPLICATIVO AND 
                    NRORG = :P_NRORG";
                    
        $stmt = $this->connection->prepare($sql);
        
        $stmt->execute(
            array(
                "IDAPLICATIVO" => $appId,
                "P_NRORG" => $this->environment->getNrorgTrab()
            )
        );
        
        $result = $stmt->fetchAll();
        
		if (isset($result[0]['LOGO'])) {
    		return $this->fileServerURL . $result[0]['LOGO'];
        } else {
            return false;
        }
    }
    
    private function getLogoAttributes(&$logoSrc, &$attrImg = 'width="200px"')
    {
        if ($logoSrc != false && $this->checkFileExists($logoSrc)) {
            $imgAttributes = getimagesize($logoSrc);
            
            if ($imgAttributes[1] > $imgAttributes[0]) {
                $attrImg = 'height="175px"';
            }
        } else {
            $logoSrc = 'https://www.teknisa.com/imagens/logo-teknisa.png';
        }
    }
    
    private function checkFileExists($url)
    {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_exec($ch);
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        
        if ($code == 200) {
            $status = true;
        } else {
            $status = false;
        }
        
        curl_close($ch);
        return $status;
    }
    
    /*********************************************************************************************/
    /**************************************** CUSTOM SMTP ****************************************/
    /*********************************************************************************************/

    public function useCustomSmtp()
    {
        if ($this->isUsingCustomSmtp) {
            $this->customSmtpSettings = $this->getCustomSmtpSettings();
        }
        
        $this->smtpUtil = new SMTP($this->customSmtpSettings);

        if (isset($this->customSmtpSettings['fromName'])) {
            $this->smtpUtil->setFromName($this->customSmtpSettings['fromName']);
        }

        if (isset($this->customSmtpSettings['from'])) {
            $this->smtpUtil->setNameReplyTo($this->customSmtpSettings['from']);
        }
    }

    public function getCustomSmtpSettings()
    {
        try{
            $filialParasupri = $this->getFilialParasupri();
            $dadosSMTPFilial = $this->getDadosSMTPFromFilial($filialParasupri);
            $settings = self::getSmtpSettingsFromAddress($dadosSMTPFilial['DSSMTPEMAILF']);

            if (isset($settings['host']) && !empty($settings['host'])) {
                if (isset($settings['port'])) {
                    $params['port'] = $settings['port'];
                }
        
                $params['host'] = $settings['host'];
                $params['from'] = $dadosSMTPFilial['DSEMAILFILI'];
                $params['fromName'] = $dadosSMTPFilial['DSEMAILFILI'];
                $params['username'] = $dadosSMTPFilial['DSLOGINSMTP'];
                $params['password'] = $this->getPassword($dadosSMTPFilial['CDSENHEMAILF']);
                
                $securityParameters = $this->getSecurityParameters($dadosSMTPFilial, $params);
                $params = array_merge($params, $securityParameters);
            
                return $params;
            }else{
                return[];
            }
        }catch (\Exception $e) {
            return[];
        }
       
    }
    
    /**
     * Recupera a senha descriptografada. Caso a senha não exista nos parâmetros,
     * retorna uma string vazia.
     * 
     * @param array $customSmtpSettings
     * 
     * @return string Contendo a senha devidamente tratada.
     */
    private function getPassword($password)
    {
        if (empty($password)) {
            return '';
        }

        return self::getDelphiPasswordEncrypted($password, false);
    }
    
    /**
     * Recupera informações de segurança para usar na conexão com o SMTP.
     * Utiliza dos parâmetros recuperados para definir as seguintes propriedades:
     * . smtpSecure:
     *      - 'tls', caso o parâmetro IDUTTLSEMAIL esteja marcado;
     *      - 'ssl', caso o parâmetro IDUTSSLEMAIL esteja marcado;
     *      - '', caso ambos os parâmetros acima estejam desmarcados;
     * . smtpAuth:
     *      - true, caso o usuário e senha estejam preenchidos;
     *      - false, caso usuário ou senha estejam em branco;
     * 
     * @param array $dadosSMTPFilial
     * @param array $customSmtpSettings
     * 
     * @return array Contendo os parâmetros de segurança.
     */
    private function getSecurityParameters($dadosSMTPFilial, $customSmtpSettings)
    {
        $params = [
            'smtpSecure' => '',
            'smtpAuth' => false
        ];
        
        if ($dadosSMTPFilial['IDUTTLSEMAIL'] == 'S') {
            $params['smtpSecure'] = 'tls';
        } else if ($dadosSMTPFilial['IDUTSSLEMAIL'] == 'S') {
            $params['smtpSecure'] = 'ssl';
        }
        
        if (!empty($customSmtpSettings['username']) && !empty($customSmtpSettings['password'])) {
            $params['smtpAuth'] = true;
        }
        
        return $params;
    }
    
    /**
     * Recupera informações do endereço de SMTP, como:
     * - host;
     * - porta;
     * 
     * @param string $url Url do servidor com ou sem porta.
     * 
     * @return array Contendo as informações encontradas.
     */
    public function getSmtpSettingsFromAddress($url)
    {
        $match = self::splitByCommonDelimiters($url);
        
        if (empty($match)) {
            throw new \Exception('Formato do endereço de SMTP inválido.');
        }
        
        if (count($match) > 1) {
            $params['port'] = $match[1];
        }
        
        $params['host'] = $match[0];
        
        return $params;
    }
        
    public function splitByCommonDelimiters($string)
    {
        $pattern = '/[:;|]+/';
        $cleanedString = self::removeAllWhitespaces($string);
        
        $splittedString = preg_split($pattern, $cleanedString);
        
        return $splittedString;
    }
    
    public function removeAllWhitespaces($string)
    {
        $pattern = '/\s+/';
        
        $cleanedString = preg_replace($pattern, '', $string);
        
        return $cleanedString;
    }

    public function getFilialParasupri()
    {
        $query = "SELECT CDFILIAL FROM PARAMGERALECOMMERCE WHERE IDFILIALPADRAOSMTP = 'S'";
                    
        $std = $this->connection->prepare($query);
        
        $std->execute();
        
        $parasupri = $std->fetch();
        
        if (empty($parasupri['CDFILIAL'])) {
            throw new \Exception("Central de compras não encontrada.");
        }
        
        return $parasupri['CDFILIAL'];
    }

    public function getCentralCompras($codigoFilial)
    {
        $query = "SELECT
                        F.CDFILIAL,
                        F.NMFILIAL,
                        F2.CDFILIAL   AS CDFILCENTCMP,
                        F2.NMFILIAL   AS NMFILCENTCMP,
                        F2.NRORG
                    FROM
                        FILIAL   F,
                        FILIAL   F2
                    WHERE
                        F.CDFILIAL = :CDFILIAL
                        AND F2.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "CDFILIAL" => $codigoFilial,
        ));
        
        return $std->fetch();
    }

    public function getDadosSMTPFromFilial($filial)
    {
        $query = "SELECT EF.CDSENHEMAILF, EF.DSSMTPEMAILF, NVL(EF.DSLOGINSMTP, EF.DSEMAILFILI) DSLOGINSMTP,
                          NVL(EF.DSEMAILFILI, EF.DSLOGINSMTP) DSEMAILFILI, EF.IDUTTLSEMAIL,
                           EF.IDUTSSLEMAIL, FI.CDFILIAL||' - '||FI.NMFILIAL AS NMFILIAL
                      FROM ENDEFILI EF INNER JOIN FILIAL FI ON EF.CDFILIAL = FI.CDFILIAL
                     WHERE EF.CDFILIAL = :P_CDFILIAL
                       AND EF.IDTPENDEFILI = 'P'";
                       
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "P_CDFILIAL" => $filial,
        ));
        
        $smtpFilial = $std->fetch();
        
        if (empty($smtpFilial)) {
            throw new \Exception("Parametrização do servidor SMTP não encontrada.");
        }
        
        return $smtpFilial;
    }

    public function getVisaoSistema()
    {
        $filialLogada = $this->environment->getCdFilial();
        $centralCompras = $this->getCentralCompras($filialLogada)['CDFILCENTCMP'];

        if ($centralCompras == $filialLogada) {
            return 'SUP';
        }

        return 'CMP';
    }
    
    /**
     * Temporário até que uma solução seja implementada no local original desta
     * função.
     * 
     * A original realiza chamada apenas nas funções GetCriptoPass e GetUnCriptoPass,
     * equivalentes ao CriptoPlus e UncriptoPlus do Delphi.
     * 
     * Porém, utilizamos as criptografias convencionais, que até então não eram
     * implementadas la Bib_html.
     * 
     * Foi incluída na 'Bib', rdm 'rdmDBGeral', na issue #238324
     * 
     * Teknisa\Libs\Util\Utilities
     */
    public static function getDelphiPasswordEncrypted($password, $isEncrypt = true, $offset = '2')
    {
        $url = self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';
        
        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );
    
        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar($offset, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
        );
    
        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUncripto';
        
        $params = array(
            'rdm'    => 'Bib.rdmDBGeral',
            'method' => $cryptType,
            'args'   => $args,
        );
    
        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }
    
        return $encryption;
    }
    
    /**
     * Temporário até que uma solução seja implementada para a função getDelphiPasswordEncrypted.
     * 
     * Teknisa\Libs\Util\Utilities
     */
    public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }
    
    /**
     * Transforma os parâmetros de um request do tipo Filter (array de array)
     * em row (array chave/valor), facilitando o acesso ao valor através do nome.
     * Ex.:
     * - Antes:  $array[0]['value'] para recuperar o valor do primeiro parâmetro.
     * - Depois: $array['columnName'] para recuperar o valor do parâmetro 'columnName'.
     * 
     * @param Zeedhi\Framework\DTO\Request\Filter $request
     * 
     * @return array Contendo o nome e valor dos parâmetros encontrados.
     */
    public function getParametersFromCondititions(\Zeedhi\Framework\DTO\Request\Filter $request)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
     
        if (empty($conditions)) return $conditions;
        
        return array_merge(...array_map(function($param) {
        	return [ $param['columnName'] => $param['value'] ];
        }, $conditions));
    }
}